"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.webpackConfig = void 0;
var path_1 = __importDefault(require("path"));
var html_webpack_plugin_1 = __importDefault(require("html-webpack-plugin"));
var webpack_bundle_analyzer_1 = require("webpack-bundle-analyzer");
var utils_1 = require("../utils");
var css_1 = require("./css");
var dev_1 = require("./dev");
var prod_1 = require("./prod");
function webpackConfig(options) {
    var isDev = process.env.NODE_ENV === 'development';
    var root = options.root, config = options.config;
    var mode = config.mode, _a = config.context, context = _a === void 0 ? root : _a, entry = config.entry, html = config.html, publicPath = config.publicPath, _b = config.dest, dest = _b === void 0 ? 'dist' : _b, externals = config.externals, alias = config.alias, _c = config.tsConfigFile, tsConfigFile = _c === void 0 ? 'tsconfig.json' : _c, extraTranspileIncludes = config.extraTranspileIncludes, configWebpack = config.configWebpack, analyzer = config.analyzer, _d = config.extarRules, extarRules = _d === void 0 ? [] : _d, _e = config.extraPlugins, extraPlugins = _e === void 0 ? [] : _e;
    var tsLoaderOptions = {
        transpileOnly: true,
        configFile: path_1.default.resolve(root, tsConfigFile)
    };
    var rules = [{
            test: /\.m?jsx?$/,
            include: [root],
            use: {
                loader: require.resolve('babel-loader'),
            }
        }, {
            test: /\.tsx?$/,
            include: [root],
            use: [{
                    loader: require.resolve('babel-loader'),
                }, {
                    loader: require.resolve('ts-loader'),
                    options: tsLoaderOptions
                }]
        }];
    if (extraTranspileIncludes === null || extraTranspileIncludes === void 0 ? void 0 : extraTranspileIncludes.length) {
        rules.push({
            test: /\.(j|t)sx?$/,
            include: (0, utils_1.makeArray)(extraTranspileIncludes),
            use: [{
                    loader: require.resolve('babel-loader'),
                }, {
                    loader: require.resolve('ts-loader'),
                    options: tsLoaderOptions
                }]
        });
    }
    var plugins = [];
    if (html) {
        for (var key in html) {
            var options_1 = html[key];
            plugins.push(new html_webpack_plugin_1.default(__assign({ filename: "".concat(key, ".html") }, options_1)));
        }
    }
    if (analyzer) {
        plugins.push(new webpack_bundle_analyzer_1.BundleAnalyzerPlugin());
    }
    var webpackConfig = {
        mode: (mode !== null && mode !== void 0 ? mode : isDev) ? 'development' : 'production',
        context: context,
        entry: entry,
        output: {
            publicPath: publicPath,
            filename: '[name].js',
            chunkFilename: '[name].js',
            path: path_1.default.resolve(root, dest),
            clean: true
        },
        externals: externals,
        resolve: {
            alias: alias,
            extensions: [
                '.js',
                '.jsx',
                '.ts',
                '.tsx',
                '.json',
                '.wasm',
                '.mjs',
            ],
            symlinks: true,
            modules: [
                'node_modules',
                path_1.default.join(__dirname, '../../node_modules')
            ]
        },
        resolveLoader: {
            modules: [
                'node_modules',
                path_1.default.join(__dirname, '../../node_modules')
            ]
        },
        optimization: {
            runtimeChunk: 'single',
            splitChunks: {
                cacheGroups: {
                    vendors: {
                        test: /\/node_modules\//,
                        name: 'vendors',
                        chunks: 'all',
                    }
                }
            }
        },
        module: {
            rules: __spreadArray(__spreadArray([], rules, true), extarRules, true),
        },
        plugins: __spreadArray(__spreadArray([], plugins, true), extraPlugins, true),
        stats: 'errors-only'
    };
    (0, css_1.cssConfig)(webpackConfig, options);
    if (configWebpack) {
        configWebpack(webpackConfig);
    }
    if (isDev) {
        (0, dev_1.devConfig)(webpackConfig, options);
    }
    else {
        (0, prod_1.prodConfig)(webpackConfig, options);
    }
    return webpackConfig;
}
exports.webpackConfig = webpackConfig;
